﻿// ============================================================================
//
// = 目录
//       chrome/realso
//
// = 文件名
//      login.js
//
// = 描述
//      处理用户登录逻辑。
//      
// = 作者
//      李杰
//      干劲 javajin@hotmail.com
//
// == 历史
//      0306：动态计算对话框位置以适应不同分辨率。@
//      0306：去掉不必要的消息响应（存在内存泄露）@
//      0306：增加记住上次登录用户而不保存密码特性@
//      0306：选择用户后，自动处理是否保存密码@
//      0306：修正已保存登录信息的用户再次登录时重复保存登录信息的Bug@
//      0306：修改RDF命名空间@
//      0306：国际化：将中文提示改为从StringBundle中加载
//      0315：代码规范化。Alpha 1.0版本形成
//
// == 待处理
//      密码加密存储
// ============================================================================

/*
  全局变量申明
*/

// 用户名
var predName = rdfs.GetResource("http://realso.cn/rdf#UserName");

// 密码
var predPwd = rdfs.GetResource("http://realso.cn/rdf#Password");

// 保存密码
var predRemPwd = rdfs.GetResource("http://realso.cn/rdf#RemPassword");

// 是否上次登录用户
var predLast = rdfs.GetResource("http://realso.cn/rdf#LastLogin");

// 服务器地址
var predServer = rdfs.GetResource("http://realso.cn/rdf#Server");

// 用户配置主语
var seqRoot = rdfs.GetResource("http://realso.cn/rdf#UserSequence");

// Bool True
var litTrue = rdfs.GetLiteral("true");

/*
  用户登录初始化
  从RDF中读取用户本地保存的登录名和密码
*/

function onLoad()
{
    try
    {
        // 窗口居中
        window.moveTo ((screen.width-document.getElementById("0").width )/2 ,(screen.height-document.getElementById("0").height )/2);
        
        // 配置文件路径
        var filepath = RSGetPath("/User.rdf","file");
        
        // 配置数据源
        var ds = rdfs.GetDataSourceBlocking(filepath);

        
        // 读取服务器IP
        var svr = ds.GetTarget(seqRoot,predServer,true);
        
        // 如果没有定义服务器，服务器为本机
        if(! svr) svr = "LOCALHOST"
        
        // 否则，从数据源读服务器
        else svr=svr.QueryInterface(CI.nsIRDFLiteral).Value;
        
        // 显示服务器
        document.getElementById("RSWLogServer").value = svr;
        
        // 临时变量
        var subIndex;
        var tmpName;
        var tmpPwd;
        var tmpRemPwd;

        //遍历RDF Datasource 如有用户名则填充menulist
        var rootCon = conunit.MakeSeq(ds,seqRoot);
        
        for(var i=1;i<=rootCon.GetCount( );i++)
        {
            // 得到一个用户
            subIndex=GetSeqItem(ds,seqRoot,i);
            
            // 用户名
            tmpName=ds.GetTarget(subIndex,predName,true).QueryInterface(CI.nsIRDFLiteral).Value;
            
            // 密码
            tmpPwd=ds.GetTarget(subIndex,predPwd,true);
            
            // 保存密码
            tmpRemPwd=ds.GetTarget(subIndex,predRemPwd,true);
            
            // 转为资源
            subIndex=subIndex.QueryInterface(CI.nsIRDFResource);
            
            // 填充menulist菜单
            var m = document.getElementById('RSWLogUsr').insertItemAt(i-1,tmpName);
            
            // 设置tooltip
            m.tooltipText=tmpName;
            
            // 在value中保存用户URI
            m.value = subIndex.Value;
            
            // 如果保存了密码
            if(tmpRemPwd)
            {
                // 密码
                tmpPwd=tmpPwd.QueryInterface(CI.nsIRDFLiteral).Value;
                
                // 是否保存密码
                tmpRemPwd=tmpRemPwd.QueryInterface(CI.nsIRDFLiteral).Value
                
                // 将密码信息保存到节点属性中
                m.setAttribute("RSRemPwd",tmpRemPwd);
                
                m.setAttribute("RSPassword",tmpPwd);
            }
            
            // 如果没有保存密码
            else
                m.setAttribute("RSRemPwd","false");
           
        }
        
        // 如果有用户
        if(rootCon.GetCount( )>0)
        {
            // 找到上次登录用户
            var v = ds.GetSource(predLast,litTrue,true);
            
            // 判断是否为空
            if(!v) return;
            
            // 得到该用户在Seq中的序号，该序号-1即是menulist中的序号
            var idx = rootCon.IndexOf(v)-1;
            
            // 选择上次登录的用户
            document.getElementById('RSWLogUsr').selectedIndex = idx;
            
            // 加载该用户的其他信息
            onSelect();
        }
   }
   catch(e)
   {
      alert("RSWLogin.xul "+"RSWLoginInit:"+e);
   }
}
    

//////////////////////////////////////
/*
    用户身份验证
*/
function onLogin()
{
    try{
        // 输入合法性
        if((document.getElementById("RSWLogUsr").value)=="" || (document.getElementById("RSWLogPwd").value)=="")
        {
            RSAlertLocal("InputPassword");
            return;
        }
        
        // 保存服务器IP
        var filepath = RSGetPath("/User.rdf","file");
        
        // 配置数据源
        var ds = rdfs.GetDataSourceBlocking(filepath);
        
        // 原来保存的服务器
        var olds = ds.GetTarget(seqRoot,predServer,true);
        
        // 用户输入的服务器
        var news = rdfs.GetLiteral(document.getElementById("RSWLogServer").value);
        
        // 修改配置文件中的服务器
        if(olds) ds.Change(seqRoot,predServer,olds,news);
        
        // 插入
        else ds.Assert(seqRoot,predServer,news,true);
        
        // 读用户名密码
        var sUser = document.getElementById("RSWLogUsr").value;
        
        // 密码
        var sPwd = document.getElementById("RSWLogPwd").value;
        
        // 建立与通讯服务器的连接
        
        // DBAccess
        var dba = GetDBA();
        
        // 连接服务器
        var b=dba.SetServer(document.getElementById("RSWLogServer").value);
        
        // 如失败
        if(!b){
            RSAlertLocal("CNCCS");
            return;
        }
        
        //验证用户登录信息
        var sys = GetSys();
        
        // 登录
        b=sys.Login(sUser,sPwd);
        
        // 成功
        if(b)
        {
        
            // 返回值
            window.arguments[0].ok=b;
            
        }
        else
        {
            // 返回错误
            window.arguments[0].ok=null;
            
            // 提示
            RSAlertLocal("EnterError");
            
            return;
        }
        
        // 用户是否在配置文件中存在
        var bOldUser = false;
        
        // 是否要求保存密码
        var bRemPwd = document.getElementById("RSWLogchkbox").checked;
        
        // 用户名Literal
        var name = rdfs.GetLiteral(sUser);
        
        // 判断输入的用户是否已在列表中存在，如果不存在，则加入到列表中
        var predID = ds.GetSource(predName,name,true);
        
        // 用户存在
        if(predID)
        {
            // 转为资源
            predID = predID.QueryInterface(CI.nsIRDFResource)
            
            bOldUser = true;
        }
        else
        {
            // 保存用户的Seq
            var rootCon = conunit.MakeSeq(ds,seqRoot);
            
            // 用户URI
            predID = rdfs.GetResource("http://realso.cn/User#"+sUser);
            
        }
        
        if(bOldUser)
        {
            // 修改是否保存密码标志
            var n = ds.GetTarget(predID,predRemPwd,true);
            
            if(n)
            {
                // 原来存了密码
                n = n.QueryInterface(CI.nsIRDFLiteral);
                
                if(! bRemPwd)
                {
                    // 删除存密码标志
                    ds.Unassert(predID,predRemPwd,n);
                    
                    // 删除已保存的密码
                    var oldp = ds.GetTarget(predID,predPwd,true)
                    
                    ds.Unassert(predID,predPwd,oldp);
                }
            }
            else if(bRemPwd)
            {
                // 原来没有存密码，现在要存
                ds.Assert(predID,predRemPwd,rdfs.GetLiteral("true"),true);
                
                ds.Assert(predID,predPwd,rdfs.GetLiteral(sPwd),true);
            }
            
        }
        else
        {
            // 这是一个新用户
            // 
            var rootCon = conunit.MakeSeq(ds,seqRoot);
            
            // 插入用户到Seq
            rootCon.AppendElement(predID);
            
            // 保存用户名
            ds.Assert(predID, predName, name, true);
            
            // 保存密码
            if(bRemPwd)
            {
                ds.Assert(predID, predPwd, rdfs.GetLiteral(sPwd), true);
                
                ds.Assert(predID, predRemPwd, rdfs.GetLiteral(bRemPwd), true);
            }
        }
        
        // 原来的最后登录用户
        var u1 = ds.GetSource(predLast,litTrue,true);
        
        // 删除原来的最后登录用户
        if(u1)
            ds.Unassert(u1,predLast,litTrue);
        
        // 新的最后登录用户
        ds.Assert(predID,predLast,litTrue,true);
        
        // 保存RDF文件
        ds.QueryInterface(CI.nsIRDFRemoteDataSource).Flush();
        
        // 关闭窗口
        window.close();
        
        return;
    }
    catch(e)
    {
        alert("RSWLogin.xul "+"RSWLogIn:"+e);
    }

}

//////////////////////////////////////////////////////////
/*
删除已经保存的登录信息
*/    
function onDelete()
{
    try
    {
        // 是否选择了用户
        if((document.getElementById("RSWLogUsr").selectedIndex) == -1 || document.getElementById("RSWLogUsr").selectedItem.label != document.getElementById("RSWLogUsr").value)
        {
            RSAlertLocal("ChoiceUser");
            return;
        }
        
        // 数据源
        var filepath = RSGetPath("/User.rdf","file");
        
        var ds = rdfs.GetDataSourceBlocking(filepath);
    
        // 用户
        var name = rdfs.GetLiteral(document.getElementById("RSWLogUsr").value);
        
        // 密码
        var pwd = rdfs.GetLiteral(document.getElementById("RSWLogUsr").selectedItem.getAttribute("RSPassword"));
        
        // 用户URI
        var id =  rdfs.GetResource(document.getElementById("RSWLogUsr").selectedItem.value);
        
        // 保存密码
        var rem =  rdfs.GetLiteral(document.getElementById("RSWLogUsr").selectedItem.getAttribute("RSRemPwd"));

        //遍历RDF Datasource Sequence 验证用户名和密码，对则删除用户名和密码
        var rootCon = conunit.MakeSeq(ds,seqRoot);
    
        // 删除Seq
        rootCon.RemoveElement(id,true);
        
        // 删除用户名
        ds.Unassert(id, predName,name);

        if(rem)
        {
            // 删除密码
            ds.Unassert(id, predPwd,pwd);
            // 删除保存密码标志
            ds.Unassert(id, predRemPwd,rem);
        }
        
        // 保存RDF
        ds.QueryInterface(CI.nsIRDFRemoteDataSource).Flush();
        
        // 删除menuitem
        document.getElementById("RSWLogUsr").removeItemAt(document.getElementById("RSWLogUsr").selectedIndex);
        
        // 清空显示
        document.getElementById("RSWLogchkbox").checked= false;
        
        document.getElementById("RSWLogPwd").value= "";
        
        document.getElementById("RSWLogUsr").value= "";
        
        // 提示成功
        RSAlertLocal("DelSuccess");
        
        return;
    }
    catch(e)
    {
       alert("RSWLogin.xul "+"RSWLogDelte:"+e);
    }

}

/////////////////////////////////////////////////////////////
/*
    依据选择的用户名，从RDF中读取密码，显示在密码输入框
*/
function onSelect()
{
    try
    {  
        // 选中的用户menuitem
        var u = document.getElementById('RSWLogUsr').selectedItem;
        
        if(! u ) return;
        
        // 是否保存密码
        if(u.getAttribute("RSRemPwd") == "true")
        {
            // 密码保存标志
            document.getElementById('RSWLogchkbox').checked = true;
            
            // 填入密码
            document.getElementById('RSWLogPwd').value = u.getAttribute("RSPassword");
        }
        else
        {
            // 密码保存标志
            document.getElementById('RSWLogchkbox').checked = false;
            
            // 填入密码
            document.getElementById('RSWLogPwd').value = "";
        }
    }
    catch(e)
    {
       alert("RSWLogin.xul "+"RSWLogmnuch:"+e);
    }
 }
