﻿// ============================================================================
//
// = 目录
//       chrome/realso
//
// = 文件名
//      dragboard.js
//
// = 描述
//      拖放处理。
//      
// = 作者
//      邓东东
//
// == 历史
//      0317：代码规范化。Alpha 1.0版本形成
//
// == 待处理
//      
// ============================================================================

/**
 * nsRSDragAndDrop - a convenience wrapper for nsTransferable, nsITransferable
 *                 and nsIDragService/nsIDragSession. 
 *
 * Use: map the handler functions to the 'ondraggesture', 'ondragover' and
 *   'ondragdrop' event handlers on your XML element, e.g.                   
 *   <xmlelement ondraggesture="nsDragAndDrop.startDrag(event, observer);"   
 *               ondragover="nsDragAndDrop.dragOver(event, observer);"      
 *               ondragdrop="nsDragAndDrop.drop(event, observer);"/>         
 *                                                                           
 *   You need to create an observer js object with the following member      
 *   functions:                                                              
 *     Object onDragStart (event)        // called when drag initiated,      
 *                                       // returns flavour list with data   
 *                                       // to stuff into transferable      
 *     void onDragOver (Object flavour)  // called when element is dragged   
 *                                       // over, so that it can perform     
 *                                       // any drag-over feedback for provided
 *                                       // flavour                          
 *     void onDrop (Object data)         // formatted data object dropped.   
 *     Object getSupportedFlavours ()    // returns a flavour list so that   
 *                                       // nsTransferable can determine
 *                                       // whether or not to accept drop. 
 **/   

var nsRSDragAndDrop = {
  
  _mDS: null,
  get mDragService()
    { try{
      if (!this._mDS) 
        {
          const kDSContractID = "@mozilla.org/widget/dragservice;1";
          const kDSIID = Components.interfaces.nsIDragService;
          this._mDS = Components.classes[kDSContractID].getService(kDSIID);
        }
        
      return this._mDS;
      }catch(e){
         alert(e);
      }
    },

  /**
   * void startDrag (DOMEvent aEvent, Object aDragDropObserver) ;
   *
   * called when a drag on an element is started.
   *
   * @param DOMEvent aEvent
   *        the DOM event fired by the drag init
   * @param Object aDragDropObserver
   *        javascript object of format described above that specifies
   *        the way in which the element responds to drag events.
   **/  
  startDrag: function (aEvent, aDragDropObserver)
    { try{ 
      if (!("onDragStart" in aDragDropObserver))
        return;
      const kDSIID = Components.interfaces.nsIDragService;
      var dragAction = { action: kDSIID.DRAGDROP_ACTION_COPY + kDSIID.DRAGDROP_ACTION_MOVE + kDSIID.DRAGDROP_ACTION_LINK };
      var transferData = { data: null };
          aDragDropObserver.onDragStart(aEvent, transferData, dragAction);
        }
      catch (e) 
        {
          alert(e);
          return;  // not a draggable item, bail!
        }

      if (!transferData.data) return;
      transferData = transferData.data;
      
      var transArray = Components.classes["@mozilla.org/supports-array;1"]
                                 .createInstance(Components.interfaces.nsISupportsArray);

      var region = null;
      if (aEvent.originalTarget.localName == "treechildren") {
        // let's build the drag region
        var tree = aEvent.originalTarget.parentNode;
        try {
          region = Components.classes["@mozilla.org/gfx/region;1"].createInstance(Components.interfaces.nsIScriptableRegion);
          region.init();
          var obo = tree.treeBoxObject;
          var bo = obo.treeBody.boxObject;
          var sel= obo.view.selection;

          var rowX = bo.x;
          var rowY = bo.y;
          var rowHeight = obo.rowHeight;
          var rowWidth = bo.width;

          //add a rectangle for each visible selected row
          for (var i = obo.getFirstVisibleRow(); i <= obo.getLastVisibleRow(); i ++)
          {
            if (sel.isSelected(i))
              region.unionRect(rowX, rowY, rowWidth, rowHeight);
            rowY = rowY + rowHeight;
          }
      
          //and finally, clip the result to be sure we don't spill over...
          region.intersectRect(bo.x, bo.y, bo.width, bo.height);
        } catch(ex) {
          dump("Error while building selection region: " + ex + "\n");
          region = null;
        }
      }

      var count = 0;
      do 
        {
          var trans = nsTransferable.set(transferData._XferID == "TransferData" 
                                         ? transferData 
                                         : transferData.dataList[count++]);
          transArray.AppendElement(trans.QueryInterface(Components.interfaces.nsISupports));
        }
      while (transferData._XferID == "TransferDataSet" && 
             count < transferData.dataList.length);
      
      try {
        this.mDragService.invokeDragSession(aEvent.target, transArray, region, dragAction.action);
      }
      catch(ex) {
        // this could be because the user pressed escape to
        // cancel the drag. even if it's not, there's not much
        // we can do, so be silent.
      }
      aEvent.stopPropagation();
    },

  /** 
   * void dragOver (DOMEvent aEvent, Object aDragDropObserver) ;
   *
   * called when a drag passes over this element
   *
   * @param DOMEvent aEvent
   *        the DOM event fired by passing over the element
   * @param Object aDragDropObserver
   *        javascript object of format described above that specifies
   *        the way in which the element responds to drag events.
   **/
  dragOver: function (aEvent, aDragDropObserver)
    { try{
      if (!("onDragOver" in aDragDropObserver)) 
        return;
      if (!this.checkCanDrop(aEvent, aDragDropObserver))
        return;
      var flavourSet = aDragDropObserver.getSupportedFlavours();
      for (var flavour in flavourSet.flavourTable)
        {
          if (this.mDragSession.isDataFlavorSupported(flavour))
            {  
              aDragDropObserver.onDragOver(aEvent, 
                                           flavourSet.flavourTable[flavour], 
                                           this.mDragSession);
              aEvent.stopPropagation();
              break;
            }
        }
      }catch(e){
          alert(e);
      }
    },

  mDragSession: null,

  /** 
   * void drop (DOMEvent aEvent, Object aDragDropObserver) ;
   *
   * called when the user drops on the element
   *
   * @param DOMEvent aEvent
   *        the DOM event fired by the drop
   * @param Object aDragDropObserver
   *        javascript object of format described above that specifies
   *        the way in which the element responds to drag events.
   **/
  drop: function (aEvent, aDragDropObserver)
    { try{
      if (!("onDrop" in aDragDropObserver))
        return;
      if (!this.checkCanDrop(aEvent, aDragDropObserver))
        return;  
      // For treechildren from RDF, it must do this.2006-10-9 
     //if('treechildren'==aEvent.target.tagName)
     //    this.mDragSession.canDrop = true;
      //  else
      //   this.mDragSession.canDrop = false;
      if (this.mDragSession.canDrop) {
        var flavourSet = aDragDropObserver.getSupportedFlavours();
        var transferData = nsTransferable.get(flavourSet, this.getDragData, true);
        // hand over to the client to respond to dropped data
        var multiple = "canHandleMultipleItems" in aDragDropObserver && aDragDropObserver.canHandleMultipleItems;
        var dropData = multiple ? transferData : transferData.first.first;
        aDragDropObserver.onDrop(aEvent, dropData, this.mDragSession);
      }
      aEvent.stopPropagation();
       
      }catch(e){
         alert(e);
      }
    },

  /** 
   * void dragExit (DOMEvent aEvent, Object aDragDropObserver) ;
   *
   * called when a drag leaves this element
   *
   * @param DOMEvent aEvent
   *        the DOM event fired by leaving the element
   * @param Object aDragDropObserver
   *        javascript object of format described above that specifies
   *        the way in which the element responds to drag events.
   **/
  dragExit: function (aEvent, aDragDropObserver)
    {  
      try{ 
      if (!this.checkCanDrop(aEvent, aDragDropObserver))
        return;
         
      if ("onDragExit" in aDragDropObserver)
        aDragDropObserver.onDragExit(aEvent, this.mDragSession);
         
      }catch(e){
          alert(e);
      }
    },  
    
  /** 
   * void dragEnter (DOMEvent aEvent, Object aDragDropObserver) ;
   *
   * called when a drag enters in this element
   *
   * @param DOMEvent aEvent
   *        the DOM event fired by entering in the element
   * @param Object aDragDropObserver
   *        javascript object of format described above that specifies
   *        the way in which the element responds to drag events.
   **/
  dragEnter: function (aEvent, aDragDropObserver)
    { 
      try{ 
      if (!this.checkCanDrop(aEvent, aDragDropObserver))
        return;
      if ("onDragEnter" in aDragDropObserver)
        aDragDropObserver.onDragEnter(aEvent, this.mDragSession);
        
      }catch(e){
         alert(e);
      }
    },  
    
  /** 
   * nsISupportsArray getDragData (Object aFlavourList)
   *
   * Creates a nsISupportsArray of all droppable items for the given
   * set of supported flavours.
   * 
   * @param FlavourSet aFlavourSet
   *        formatted flavour list.
   **/  
  getDragData: function (aFlavourSet)
    { 
      try{
      var supportsArray = Components.classes["@mozilla.org/supports-array;1"]
                                    .createInstance(Components.interfaces.nsISupportsArray);

      for (var i = 0; i < nsRSDragAndDrop.mDragSession.numDropItems; ++i)
        {
          var trans = nsTransferable.createTransferable();
          for (var j = 0; j < aFlavourSet.flavours.length; ++j)
            trans.addDataFlavor(aFlavourSet.flavours[j].contentType);
          nsRSDragAndDrop.mDragSession.getData(trans, i);
          supportsArray.AppendElement(trans);
        }
      return supportsArray;
      }catch(e){
          alert(e);
      }
    },

  /** 
   * Boolean checkCanDrop (DOMEvent aEvent, Object aDragDropObserver) ;
   *
   * Sets the canDrop attribute for the drag session.
   * returns false if there is no current drag session.
   *
   * @param DOMEvent aEvent
   *        the DOM event fired by the drop
   * @param Object aDragDropObserver
   *        javascript object of format described above that specifies
   *        the way in which the element responds to drag events.
   **/
  checkCanDrop: function (aEvent, aDragDropObserver)
    { try{
      if (!this.mDragSession) 
        this.mDragSession = this.mDragService.getCurrentSession();
      if (!this.mDragSession) 
        return false;
      this.mDragSession.canDrop = this.mDragSession.sourceNode != aEvent.target;
      
      if('treechildren'==aEvent.target.tagName)
         this.mDragSession.canDrop = true;
      
      if ("canDrop" in aDragDropObserver)
        this.mDragSession.canDrop &= aDragDropObserver.canDrop(aEvent, this.mDragSession);
      return true;
      }catch(e){
         alert(e);
      }
    } 
};




var listObserver = {
 onDragStart: function (evt,transferData,action)
 { 
  try{
      if('treechildren'==evt.target.tagName)
      {
          var treeEvt = evt.target.parentNode;
          //alert(treeEvt.view.getCellText(treeEvt.currentIndex,treeEvt.columns[0]));
          var tempEmpty = 'noele';
          tempEmpty = treeEvt.view.getCellText(treeEvt.currentIndex,treeEvt.columns[0]);
          //var tempEmpty = 'button';
          transferData.data=new TransferData(); 
          transferData.data.addDataForFlavour("text/unicode", tempEmpty); 
      }
  }catch(e){
     alert(e);
  }
 } 
}; 

var boardObserver = { 
getSupportedFlavours : function () 
{
 try{
     var flavours = new FlavourSet(); 
     flavours.appendFlavour("text/unicode"); 
     return flavours; 
 }catch(e){
    alert(e);
 }
}, 
 
onDragOver: function (evt,flavour,session)
{
 try{
     // This sentence have to be executed
   if('treechildren'==evt.target.tagName)
   {
     session.canDrop = true;
     //  else
     //    session.canDrop = false;
     var treeEvt = evt.target.parentNode;
     var tree = treeEvt;
     var currentIndex = tree.treeBoxObject.getRowAt(evt.screenX, evt.screenY)-1;
     var c = tree.currentIndex;
     var rs = tree.builderView.getResourceAtIndex(c); 
     var id = rs.Value;
              
     // 取得要删除的节点的workspacenode对象                    
     delnode1 = g_w.AllNodes.getValue(id);
           
     delnode = delnode1.QueryInterface(CI.nsIRSWorkspaceNode); 
     
     var current_rs = tree.builderView.getResourceAtIndex(currentIndex); 
     var current_id = current_rs.Value;
     //alert(current_id);
     movedNode = g_w.AllNodes.getValue(current_id);     
     // called XPCOM function
     var oldParent = delnode.Parent.ID;
     var temp_node = movedNode;
     while((null!=temp_node)&&(null!=temp_node.Parent))
     {   //alert(temp_node.ID+ '  '+delnode.ID);
        if(temp_node.ID==delnode.ID)
        { 
         session.canDrop = false;
        }
        temp_node = temp_node.Parent;
     }
   }
 }catch(e){
   alert(e);
 }
}, 

onDrop: function (evt,dropdata,session)
{   
 try{  
     netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
     
     if('treechildren'==evt.target.tagName)
     { 
         if(dropdata.data!="")
         {  
            var delnode1;
            var movedNode;
            var ds;
            var tds;
            var tree;
            var current_rs;
            var current_id
            var treeEvt = evt.target.parentNode;
           
            tree = treeEvt;
            tds = tree.database.GetDataSources();
            ds=tds.getNext();
            var c = tree.currentIndex;
            
            if(0<=c)
            {  
                var s = document.getElementById("strings");
                var s1 = s.getString("FileMoveConfirm");
                var s2 = s.getString("Yes");
                var s3 = s.getString("No");
                var s4 = s.getString("Information");
               if(RSConfirm(s1,s4,s2,s3))
                {
                var rs = tree.builderView.getResourceAtIndex(c); 
                var id = rs.Value;
              
                // 取得要删除的节点的workspacenode对象                    
                delnode1 = g_w.AllNodes.getValue(id);
              
                delnode = delnode1.QueryInterface(CI.nsIRSWorkspaceNode); 
             
                {  
                    // 取得要移动到的节点
                    var currentIndex = tree.treeBoxObject.getRowAt(evt.screenX, evt.screenY)-1;
                    current_rs = tree.builderView.getResourceAtIndex(currentIndex); 
                    current_id = current_rs.Value;
                    movedNode = g_w.AllNodes.getValue(current_id);     
                    // called XPCOM function
                    var oldParent = delnode.Parent.ID;
                    var temp_node = movedNode;
                    delnode.ChangeParent(movedNode);
                    
                    
                    // step 1, removed selected node
                    var rdf = CC['@mozilla.org/rdf/rdf-service;1'].getService(Components.interfaces.nsIRDFService);
                    var cont = CC['@mozilla.org/rdf/container-utils;1'].getService(CI.nsIRDFContainerUtils);
                    ds = ds.QueryInterface(CI.nsIRDFDataSource);
                    // 在内存中的数据源中删除要删除的节点
                    var oldRes = rdf.GetResource(oldParent);
                    var delRes = rdf.GetResource(delnode.ID);
                    var oldSeq = cont.MakeSeq(ds,oldRes);
                    oldSeq.RemoveElement(delRes,true);
                    //tree.builder.rebuild(); 
                    // step 2, added selected node
                    // 把新添加的节点加入到RDF数据源中
                    var newRes = rdf.GetResource(current_id);
                    var newSeq = cont.MakeSeq(ds,newRes);
                    newSeq.AppendElement(delRes,true);

                    tree.builder.rebuild(); 
                } // if(delnode.Status==0)
              }
            } // if(0<=c)
         } // if(dropdata.data!="")
      } // if('treechildren'==evt.target.tagName)
      
 }catch(e){
    alert(e);
 }
}
};